

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/push/services/destination.proto, blickfeld/push/services/health.proto, blickfeld/push/services/push.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    config as _config__,
    data as _data__,
)


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class DestinationValidateRequest(betterproto.Message):
    """request to validate a configured destination"""

    destination: "_config__.Destination" = betterproto.message_field(1)
    """the current (possibly incomplete) configuration entered"""


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """request get the health of the configured publisher"""

    health: "_data__.Health" = betterproto.message_field(2)
    """health of push configurations"""


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """request watch the health of the configured publisher"""

    health: "_data__.Health" = betterproto.message_field(2)
    """health of push configurations"""


@dataclass(eq=False, repr=False)
class PushStoreRequest(betterproto.Message):
    """request to add a push configuration"""

    uuid: str = betterproto.string_field(1)
    """the uuid of the configuration (has to be generated on client side!)"""

    push_config: "_config__.Push" = betterproto.message_field(2)
    """push configuration to add"""


@dataclass(eq=False, repr=False)
class PushDeleteRequest(betterproto.Message):
    """request to delete a push configuration"""

    uuid: str = betterproto.string_field(1)
    """configuration uuid to delete"""


@dataclass(eq=False, repr=False)
class PushListResponse(betterproto.Message):
    """request to list all configured push configurations"""

    push_configs: Dict[str, "_config__.Push"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """current snapshot of configured push configurations"""


@dataclass(eq=False, repr=False)
class PushWatchResponse(betterproto.Message):
    """request to watch all configured push configurations"""

    push_configs: Dict[str, "_config__.Push"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """current snapshot of configured push configurations"""


@dataclass(eq=False, repr=False)
class PushEnableRequest(betterproto.Message):
    """request to enable the push configuration with the given uuid"""

    uuid: str = betterproto.string_field(1)
    """configuration uuid to enable"""


@dataclass(eq=False, repr=False)
class PushDisableRequest(betterproto.Message):
    """request to disable the push configuration with the given uuid"""

    uuid: str = betterproto.string_field(1)
    """configuration uuid to disable"""


class Destination(betterproto.ServiceStub):
    """An gRPC service to configure multiple push settings"""

    async def async_validate(
        self,
        *,
        destination: "_config__.Destination" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Setting validation"""

        request = DestinationValidateRequest()
        if destination is not None:
            request.destination = destination

        return await self._unary_unary(
            "/blickfeld.push.services.Destination/Validate",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def validate(
        self,
        *,
        destination: "_config__.Destination" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Setting validation"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_validate(
                destination=destination,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Health(betterproto.ServiceStub):
    """An gRPC service to request the status of push configurations"""

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Get Status for all configured push configurations"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.push.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Get Status for all configured push configurations"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """Watch status changes for configured push configurations"""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.push.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """Watch status changes for configured push configurations"""

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class Push(betterproto.ServiceStub):
    """An gRPC service to configure multiple push settings"""

    async def async_store(
        self,
        *,
        uuid: str = "",
        push_config: "_config__.Push" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store (Add or Update a push configuration)"""

        request = PushStoreRequest()
        request.uuid = uuid
        if push_config is not None:
            request.push_config = push_config

        return await self._unary_unary(
            "/blickfeld.push.services.Push/Store",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def store(
        self,
        *,
        uuid: str = "",
        push_config: "_config__.Push" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store (Add or Update a push configuration)"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_store(
                uuid=uuid,
                push_config=push_config,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_delete(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete an existing push configuration"""

        request = PushDeleteRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.push.services.Push/Delete",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def delete(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete an existing push configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_delete(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PushListResponse":
        """List configured push configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.push.services.Push/List",
            request,
            PushListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PushListResponse":
        """List configured push configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["PushWatchResponse"]:
        """Watch list of configured push configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.push.services.Push/Watch",
            request,
            PushWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["PushWatchResponse"]:
        """Watch list of configured push configuration"""

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_enable(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Enable a configured Push"""

        request = PushEnableRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.push.services.Push/Enable",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def enable(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Enable a configured Push"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_enable(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_disable(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Disable a configured Push"""

        request = PushDisableRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.push.services.Push/Disable",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def disable(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Disable a configured Push"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_disable(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



# Increase message size limit to allow large point cloud transfers
MAX_MESSAGE_LENGTH = 32 * 1024 * 1024
OPTIONS = [
    ("grpc.max_send_message_length", MAX_MESSAGE_LENGTH),
    ("grpc.max_receive_message_length", MAX_MESSAGE_LENGTH),
]





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/toolkit/services/preset.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import config as _config__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class PresetSetRequest(betterproto.Message):
    """Request for setting the preset"""

    preset: "_config__.Preset" = betterproto.message_field(1)
    """Preset data"""


@dataclass(eq=False, repr=False)
class PresetGetResponse(betterproto.Message):
    """Response when getting the preset"""

    preset: "_config__.Preset" = betterproto.message_field(1)
    """Preset data"""


class Preset(betterproto.ServiceStub):
    """This service provided basic information about the configured preset"""

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PresetGetResponse":
        """Get the configured preset"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.toolkit.services.Preset/Get",
            request,
            PresetGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PresetGetResponse":
        """Get the configured preset"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_set(
        self,
        *,
        preset: "_config__.Preset" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set the selected preset"""

        request = PresetSetRequest()
        if preset is not None:
            request.preset = preset

        return await self._unary_unary(
            "/blickfeld.toolkit.services.Preset/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        preset: "_config__.Preset" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set the selected preset"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                preset=preset,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/config/services/storage.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class Storage(betterproto.ServiceStub):
    """
    The Storage service provides access to the key / value based configuration
    of the device.
    """

    async def async_factory_reset(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Delete all user generated settings, log files and cached data
        WARNING: All non-factory provided settings in the device configuration
        will be permanently lost.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.config.services.Storage/FactoryReset",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def factory_reset(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Delete all user generated settings, log files and cached data
        WARNING: All non-factory provided settings in the device configuration
        will be permanently lost.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_factory_reset(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/video/services/health.proto, blickfeld/video/services/visualization.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    config as _config__,
    data as _data__,
)


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """Response to Health Get request"""

    health: "_data__.Health" = betterproto.message_field(1)
    """Health state"""


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """Response to Health Watch request"""

    health: "_data__.Health" = betterproto.message_field(1)
    """Health state"""


@dataclass(eq=False, repr=False)
class VisualizationGetRequest(betterproto.Message):
    """Get Visualization Request"""

    pass


@dataclass(eq=False, repr=False)
class VisualizationGetResponse(betterproto.Message):
    """Get Visualization response"""

    configuration: "_config__.Visualization" = betterproto.message_field(1)
    """visualization configuration"""


@dataclass(eq=False, repr=False)
class VisualizationSetRequest(betterproto.Message):
    """Set Visualization request"""

    configuration: "_config__.Visualization" = betterproto.message_field(2)
    """visualization configuration"""


class Health(betterproto.ServiceStub):
    """
    The health service provides methods to monitor operational status of the
    video module
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health status of the video module"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.video.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health status of the video module"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        Continously returns the current health status of the video module
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.video.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        Continously returns the current health status of the video module
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class Visualization(betterproto.ServiceStub):
    """
    Service allows to change the visuals of RTSP video stream served by Qb2.
    Changes can be applicable to the elements of the observed scene frame (e.g.
    background, objects) and to extra overlay elements (e.g. logo branding,
    time, clock, background color)
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["VisualizationGetResponse"]:
        """Gets named visual configuration"""

        request = VisualizationGetRequest()

        async for response in self._unary_stream(
            "/blickfeld.video.services.Visualization/Get",
            request,
            VisualizationGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["VisualizationGetResponse"]:
        """Gets named visual configuration"""

        loop = asyncio.get_event_loop()
        ait = self.async_get(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_set(
        self,
        *,
        configuration: "_config__.Visualization" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Applies visualization configuration to the video stream"""

        request = VisualizationSetRequest()
        if configuration is not None:
            request.configuration = configuration

        return await self._unary_unary(
            "/blickfeld.video.services.Visualization/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        configuration: "_config__.Visualization" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Applies visualization configuration to the video stream"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                configuration=configuration,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/toolkit/config/preset.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ... import base as __base__
from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Preset(betterproto.Message):
    """
    A preset combines various settings relevant for a specific use-case. It
    indicates the type of application and its configuration in which the device
    is being used. Presets are always available for a specific license (e.g.
    only for QbProtect). The specific settings are not stored as part of this
    message.
    """

    name: str = betterproto.string_field(1)
    """Name of the preset."""

    details: str = betterproto.string_field(2)
    """Detailed explanation of the preset use-case."""

    required_license: "__base__.ProductVariant" = betterproto.enum_field(3)
    """Product variant for which the preset is licensed and intended."""

    version: "__base_data__.Version" = betterproto.message_field(4)
    """Version of the preset."""

    metadata: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(100)
    """Arbitrary metadata storage for client applications."""



import sys

sys.path.insert(0, "/usr/lib")

import asyncio
import logging
import time
from collections.abc import AsyncGenerator, AsyncIterator, Callable

from google.protobuf.any_pb2 import Any
from grpc.aio import AioRpcError, ServicerContext
from grpc_interceptor import AsyncServerInterceptor

from blickfeld.base.grpc.base_error import BaseError

logger = logging.getLogger("server")


class AioServerInterceptor(AsyncServerInterceptor):
    async def intercept(
        self,
        method: Callable,
        request_or_iterator: Any,
        context: ServicerContext,
        method_name: str,
    ) -> AsyncGenerator[Any] | Any | None:
        start_ts = time.monotonic()
        try:
            response_or_iterator = method(request_or_iterator, context)
            if not hasattr(response_or_iterator, "__aiter__"):
                # Unary, just await and return the response
                response = await response_or_iterator
                logger.info(f"Request: {method_name} [{int((time.monotonic() - start_ts) * 1e3)}ms]")
                return response
        except (BaseError, AioRpcError) as e:
            e = BaseError.create_from_exception(e)
            logger.warning(f"Request: {method_name} [{int((time.monotonic() - start_ts) * 1e3)}ms, {e.code.name}]")
            await context.abort(e.code, e.message or "")
            return None

        logger.info(f"Stream request: {method_name}")
        # Server streaming responses, delegate to an async generator helper.
        # Note that we do NOT await this.
        return self._intercept_streaming(method_name, start_ts, response_or_iterator, context)

    async def _intercept_streaming(
        self, method_name: str, start_ts: float, iterator: AsyncIterator[Any], context: ServicerContext
    ) -> AsyncGenerator[Any]:
        try:
            async for r in iterator:
                yield r
        except (BaseError, AioRpcError) as e:
            e = BaseError.create_from_exception(e)
            logger.warning(
                f"Stream request: {method_name} [{int((time.monotonic() - start_ts) * 1e3)}ms, {e.code.name}]"
            )
            await context.abort(e.code, e.message or "")
        except asyncio.CancelledError:
            logger.info(f"Stream request: {method_name} [{int((time.monotonic() - start_ts) * 1e3)}ms]")
            raise





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/beam_deflection_control/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """
    Health message containing information about operation, status and error of
    a beam deflection controller.
    """

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is OK."""

    stats: "HealthStats" = betterproto.message_field(6)
    """Stats giving basic information about the mirror operation"""


@dataclass(eq=False, repr=False)
class HealthStats(betterproto.Message):
    """
    This message defines the stats that can be retrieved from a beam deflection
    controller.
    """

    time: float = betterproto.float_field(1)
    """Time in seconds since last operation start."""

    control_effort: float = betterproto.float_field(9)
    """
    Control effort usage. Relative value indicating how much of the maximum
    available effort is currently used to control the deflection mirror.
    """





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/video/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Describes the health of the video module"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level health state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for a given state. Only set if state is not STATE_OK"""

    clients: Dict[str, "HealthClient"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Served clients (key is the ip:port)"""

    performance: "HealthPerformance" = betterproto.message_field(4)
    """The performance of the video processing pipeline"""


@dataclass(eq=False, repr=False)
class HealthClient(betterproto.Message):
    """Describes the RTSP client served by the video server"""

    uptime: int = betterproto.uint32_field(1)
    """Time since this RTSP session has been active"""


@dataclass(eq=False, repr=False)
class HealthPerformance(betterproto.Message):
    """The performance of the video processing pipeline"""

    fetch_frequency: float = betterproto.float_field(1)
    """rate at which point cloud is fetched from the source (percept)"""

    render_frequency: float = betterproto.float_field(2)
    """rate at which point cloud is rendered"""

    push_frequency: float = betterproto.float_field(3)
    """rate at which point cloud is pushed to the sink (video pipeline)"""



from setuptools import find_packages, setup

setup(
    name="blickfeld_qb2",
    version="2.11.6",
    author="Blickfeld GmbH",
    author_email="opensource@blickfeld.com",
    url="https://github.com/Blickfeld/blickfeld-qb2",
    description="Python package to communicate securely with Qb2 LiDAR devices of the Blickfeld GmbH",
    packages=find_packages(),
    install_requires=[
        "numpy",
        # Betterproto dependencies
        "grpclib >= 0.4.1",
        "python-dateutil >= 2.8",
        "dataclasses >= 0.7;python_version<'3.7'"
	],
)






# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/diagnostics/config/log.proto, blickfeld/diagnostics/config/self_test.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class LogTimeDirection(betterproto.Enum):
    """The direction in time in which the service provides new log entries"""

    TIME_DIRECTION_UNSPECIFIED = 0
    """No direction is specified"""

    TIME_DIRECTION_FORWARD = 1
    """
    Streaming direction forward in time. The log entries are streamed towards
    the tail of the journal. When the tail is reached, the stream continues
    waiting for newly written logs.
    """

    TIME_DIRECTION_BACKWARD = 2
    """
    Streaming direction backward in time. The log entries are streamed towards
    the head of the journal. When the head is reached, the stream terminates.
    """


class LogFilterSoftwareModule(betterproto.Enum):
    """
    This field specifies the list of software modules that can be accessed via
    the log service.
    """

    SOFTWARE_MODULE_UNSPECIFIED = 0
    """No module is selected"""

    SOFTWARE_MODULE_BLICKFELD_PROXY = 1
    """
    gRPC proxy where all clients requests are handled. Helps to review & audit
    calls of client applications.
    """

    SOFTWARE_MODULE_BLICKFELD_FLOW = 2
    """
    Logs of the embedded node-red instance. Helps to trace & understand
    blickfeld flow behaviourals or crashes.
    """

    SOFTWARE_MODULE_BLICKFELD_DIAGNOSTICS = 3
    """
    Logs of the global diagnostics module. Helps to trace & understand system
    health changes.
    """


class SelfTest(betterproto.Enum):
    """
    List of available self tests   NOTE: The test list is still incomplete.
    """

    SELF_TEST_UNSPECIFIED = 0
    """No test is specified."""

    SELF_TEST_POINT_CLOUD = 1
    """
    Simple test which starts up the device, requests a point cloud, validates
    its properties and stops the device.
    """

    SELF_TEST_CONNECTION = 2
    """
    Simple connection tests to basic peripherals: PCIe, laser, detector,
    vertical BDC & horizontal BDC. It performs version requests to the modules.
    """

    SELF_TEST_BEAM_DEFLECTION_UNIT = 3
    """Simple check for basic functionality of the beam deflection units"""


@dataclass(eq=False, repr=False)
class Log(betterproto.Message):
    """
    This message aggregates the configurations related to the log service.
    """

    pass


@dataclass(eq=False, repr=False)
class LogFilter(betterproto.Message):
    """Filter configuration for filtering the requested log entries."""

    pass



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: google/protobuf/compiler/plugin.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase


class CodeGeneratorResponseFeature(betterproto.Enum):
    FEATURE_NONE = 0
    FEATURE_PROTO3_OPTIONAL = 1


@dataclass(eq=False, repr=False)
class Version(betterproto.Message):
    """The version number of protocol compiler."""

    major: int = betterproto.int32_field(1)
    minor: int = betterproto.int32_field(2)
    patch: int = betterproto.int32_field(3)
    # A suffix for alpha, beta or rc release, e.g., "alpha-1", "rc2". It should
    # be empty for mainline stable releases.
    suffix: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CodeGeneratorRequest(betterproto.Message):
    """An encoded CodeGeneratorRequest is written to the plugin's stdin."""

    # The .proto files that were explicitly listed on the command-line.  The code
    # generator should generate code only for these files.  Each file's
    # descriptor will be included in proto_file, below.
    file_to_generate: List[str] = betterproto.string_field(1)
    # The generator parameter passed on the command-line.
    parameter: str = betterproto.string_field(2)
    # FileDescriptorProtos for all files in files_to_generate and everything they
    # import.  The files will appear in topological order, so each file appears
    # before any file that imports it. protoc guarantees that all proto_files
    # will be written after the fields above, even though this is not technically
    # guaranteed by the protobuf wire format.  This theoretically could allow a
    # plugin to stream in the FileDescriptorProtos and handle them one by one
    # rather than read the entire set into memory at once.  However, as of this
    # writing, this is not similarly optimized on protoc's end -- it will store
    # all fields in memory at once before sending them to the plugin. Type names
    # of fields and extensions in the FileDescriptorProto are always fully
    # qualified.
    proto_file: List[
        "betterproto_lib_google_protobuf.FileDescriptorProto"
    ] = betterproto.message_field(15)
    # The version number of protocol compiler.
    compiler_version: "Version" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CodeGeneratorResponse(betterproto.Message):
    """The plugin writes an encoded CodeGeneratorResponse to stdout."""

    # Error message.  If non-empty, code generation failed.  The plugin process
    # should exit with status code zero even if it reports an error in this way.
    # This should be used to indicate errors in .proto files which prevent the
    # code generator from generating correct code.  Errors which indicate a
    # problem in protoc itself -- such as the input CodeGeneratorRequest being
    # unparseable -- should be reported by writing a message to stderr and
    # exiting with a non-zero status code.
    error: str = betterproto.string_field(1)
    # A bitmask of supported features that the code generator supports. This is a
    # bitwise "or" of values from the Feature enum.
    supported_features: int = betterproto.uint64_field(2)
    file: List["CodeGeneratorResponseFile"] = betterproto.message_field(15)


@dataclass(eq=False, repr=False)
class CodeGeneratorResponseFile(betterproto.Message):
    """Represents a single generated file."""

    # The file name, relative to the output directory.  The name must not contain
    # "." or ".." components and must be relative, not be absolute (so, the file
    # cannot lie outside the output directory).  "/" must be used as the path
    # separator, not "\". If the name is omitted, the content will be appended to
    # the previous file.  This allows the generator to break large files into
    # small chunks, and allows the generated text to be streamed back to protoc
    # so that large files need not reside completely in memory at one time.  Note
    # that as of this writing protoc does not optimize for this -- it will read
    # the entire CodeGeneratorResponse before writing files to disk.
    name: str = betterproto.string_field(1)
    # If non-empty, indicates that the named file should already exist, and the
    # content here is to be inserted into that file at a defined insertion point.
    # This feature allows a code generator to extend the output produced by
    # another code generator.  The original generator may provide insertion
    # points by placing special annotations in the file that look like:
    # @@protoc_insertion_point(NAME) The annotation can have arbitrary text
    # before and after it on the line, which allows it to be placed in a comment.
    # NAME should be replaced with an identifier naming the point -- this is what
    # other generators will use as the insertion_point.  Code inserted at this
    # point will be placed immediately above the line containing the insertion
    # point (thus multiple insertions to the same point will come out in the
    # order they were added). The double-@ is intended to make it unlikely that
    # the generated code could contain things that look like insertion points by
    # accident. For example, the C++ code generator places the following line in
    # the .pb.h files that it generates:   //
    # @@protoc_insertion_point(namespace_scope) This line appears within the
    # scope of the file's package namespace, but outside of any particular class.
    # Another plugin can then specify the insertion_point "namespace_scope" to
    # generate additional classes or other declarations that should be placed in
    # this scope. Note that if the line containing the insertion point begins
    # with whitespace, the same whitespace will be added to every line of the
    # inserted text.  This is useful for languages like Python, where indentation
    # matters.  In these languages, the insertion point comment should be
    # indented the same amount as any inserted code will need to be in order to
    # work correctly in that context. The code generator that generates the
    # initial file and the one which inserts into it must both run as part of a
    # single invocation of protoc. Code generators are executed in the order in
    # which they appear on the command line. If |insertion_point| is present,
    # |name| must also be present.
    insertion_point: str = betterproto.string_field(2)
    # The file contents.
    content: str = betterproto.string_field(15)
    # Information describing the file content being inserted. If an insertion
    # point is used, this information will be appropriately offset and inserted
    # into the code generation metadata for the generated files.
    generated_code_info: "betterproto_lib_google_protobuf.GeneratedCodeInfo" = (
        betterproto.message_field(16)
    )


import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/secure/data/health.proto, blickfeld/secure/data/license.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ... import base as __base__
from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Message representing health"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Only set if state is not STATE_OK."""

    user_management: "HealthUserManagement" = betterproto.message_field(3)
    """State of User-Management feature"""


@dataclass(eq=False, repr=False)
class HealthUserManagement(betterproto.Message):
    """State message for User-Management feature"""

    needs_activation: bool = betterproto.bool_field(1)
    """
    Set to `true` if user management needs manual activation. This is only set
    when a device can migrate to user management from a previous state.
    """

    state_factory: bool = betterproto.bool_field(2)
    """set to true if only default account with the default name exist."""


@dataclass(eq=False, repr=False)
class License(betterproto.Message):
    """Message representing the configured license"""

    product_variant: "__base__.ProductVariant" = betterproto.enum_field(1)
    """License product variant"""

    metadata: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(2)
    """License metadata"""

    serial_number: str = betterproto.string_field(3)
    """License serial number"""



from grpclib import GRPCError, Status
from blickfeld_qb2.base.grpc.channel import Channel

from blickfeld_qb2.secure.services import Session, Authentication, AuthenticationLoginRequestApplication


class TokenFactory:
    """Callable Token Factory which holds the current JWT access token.

    The token is checked for validity before returning it by requesting the current session-nonce.
    The token is re-generated by authenticating with the application-key-secret when it is expired or unset.

    Example Usage with `blickfeld_qb2` public API module:

        import blickfeld_qb2

        # inject `blickfeld_qb2.TokenFactory`-instance into grpc-channel for authentication
        channel = blickfeld_qb2.Channel(
            fqdn_or_ip=my_qb2_fqdn_or_ip,
            token=blickfeld_qb2.TokenFactory(application_key_secret="my-application-key")
        )

        # get session nonce
        session_service = blickfeld_qb2.secure.services.Session(channel=channel)
        nonce = session_service.get_nonce()
        print(f"session-nonce: {nonce}")

        # get account information
        account_service = blickfeld_qb2.secure.services.Account(channel=channel)
        account = account_service.get()
        print(f"account: {account}")

    """

    current_token: str = None

    def __init__(self, application_key_secret: str) -> None:
        self.application_key_secret = application_key_secret

    async def __call__(self, channel: Channel) -> str:
        """Awaitable callable interface"""
        return await self.create_or_renew_token(channel)

    async def create_or_renew_token(self, channel: Channel) -> str:
        """Awaitable handler which is used for automatic token generation with Channel from blickfeld_qb2.base."""
        if self.current_token:
            # Check if current token is still valid
            try:
                async with channel.clone(token=self.current_token) as cloned_channel:
                    _ = await Session(channel=cloned_channel).async_get_nonce()
            except GRPCError as ex:
                if ex.status == Status.PERMISSION_DENIED:
                    self.current_token = None
                else:
                    raise

            # Token is still valid, return it
            if self.current_token:
                return self.current_token

        # Authenticate to renew token
        async with channel.clone(token=None) as cloned_channel:
            login_response = await Authentication(channel=cloned_channel).async_login(
                application=AuthenticationLoginRequestApplication(key=self.application_key_secret)
            )
            self.current_token = login_response.token

        return self.current_token
